var _ = require('lodash'),
	mongoose = require('mongoose'),
	Schema = mongoose.Schema,
	Promise = mongoose.Promise,
	ObjectId = require('mongodb').ObjectID,
	moment = require('moment');

exports = module.exports = {
	getModel: function(mongoose) {
		"use strict";

		var collectionName = 'Data';
		var modelName = 'BodyWeight';
		var dataTypeName = 'body-weight';

		return (function() {
			var Model;

			if (_.isUndefined(Model)) {

				var bodyWeightSchema = new Schema({
					person: {type: Schema.ObjectId, required: true},
					name: {type: String, required: true},
					date: {type: Date, required: true},
					active: {type: Boolean, required: true},
					value: {type: Number, min: 0, max: 600, required: true},
					updatedOn: Date
				}, {
					strict: true
				});

				bodyWeightSchema.statics.getCollectionName = function getCollectionName() {
					return collectionName;
				};

				bodyWeightSchema.statics.getModelName = function getModelName() {
					return modelName;
				};

				bodyWeightSchema.statics.getDataTypeName = function getDataTypeName() {
					return dataTypeName;
				};

				bodyWeightSchema.statics.deleteByDateRange = function deleteByDateRange(personId, startDate, endDate) {
					if (!_.isDate(startDate)) { startDate = new Date(startDate); }
					if (!_.isDate(endDate)) { endDate = new Date(endDate); }
					if (_.isString(personId)) { personId = ObjectId.createFromHexString(personId); }

					return Model.update({
						name: Model.getDataTypeName(),
						person: personId,
						active: true,
						date: { $gte: startDate, $lte: endDate }
					}, {
						active: false,
						updatedOn: Date.now()
					}, {
						multi: true
					}).exec();
				};

				bodyWeightSchema.statics.deleteAllOnDay = function(personId, date) {
					var startDate = moment.utc(date).clone().startOf('day');
					var endDate = moment.utc(date).clone().endOf('day');
					return Model.deleteByDateRange(personId, startDate, endDate);
				};

				bodyWeightSchema.statics.getByDateRange = function getByDateRange(personId, startDate, endDate) {
					if (!_.isDate(startDate)) { startDate = new Date(startDate); }
					if (!_.isDate(endDate)) { endDate = new Date(endDate); }
					if (_.isString(personId)) { personId = ObjectId.createFromHexString(personId); }

					return Model.find({
						name: Model.getDataTypeName(),
						person: personId,
						active: true,
						date: { $gte: startDate, $lte: endDate}
					}).exec();
				};

				bodyWeightSchema.statics.getAllOnDay = function(personId, date) {
					var startDate = moment.utc(date).clone().startOf('day');
					var endDate = moment.utc(date).clone().endOf('day');
					return Model.getByDateRange(personId, startDate, endDate);
				};

				bodyWeightSchema.statics.add = function add(personId, weight, date) {
					if (!_.isDate(date)) { date = new Date(date); }
					if (_.isString(personId)) { personId = ObjectId.createFromHexString(personId); }

					var promise = new Promise();

					var weightDocument = new Model({
						'person': personId,
						'name': Model.getDataTypeName(),
						'value': weight,
						'date': date,
						'active': true
					});

					weightDocument.save(function(error) {
						promise.resolve(error);
					});
					return promise;
				};

				bodyWeightSchema.statics.getById = function getById(id) {
					return Model.findOne({
						_id: id,
						active: true
					}).exec();
				};

				bodyWeightSchema.statics.deleteById = function (id) {
					return Model.findByIdAndUpdate(id, {
						active: false,
						updatedOn: Date.now()
					}).exec();
				};

				mongoose.connection.collection(collectionName);
				Model = mongoose.model(modelName, bodyWeightSchema, collectionName);
			}
			return Model;
		}());
	}
};



