describe('Body Weight', function() {

	var	bodyWeight = require('../lib/body-weight'),
		person = require('../lib/person'),
		mongoose = require('mongoose'),
		expect = require('chai').expect,
		settings = require('./test-database-configuration');

	var connectionString = 'mongodb://' + settings.databaseServer + ':' + settings.databasePort + '/' + settings.databaseName;

	var BodyWeightModel = bodyWeight.getModel(mongoose);

	afterEach(function(done) {
		mongoose.disconnect(function(){done();});
	});

	beforeEach(function() {
		mongoose.connect(connectionString);
	});

	it('should get the collection name', function() {
		expect(BodyWeightModel.getCollectionName()).to.equal('Data');
	});

	it('should get the model name', function() {
		expect(BodyWeightModel.getModelName()).to.equal('BodyWeight');
	});

	describe('weights by date range', function() {
		var startDate = '2012-02-01T00:00:00.000Z';
		var weightOneDate = '2012-02-01T12:22:33.444Z';
		var weightOneValue = 123;
		var weightTwoDate = '2012-02-01T14:33:11.321Z';
		var weightTwoValue = 213.32;
		var endDate = '2012-02-01T23:59:59.999Z';
		var personId = '5170ab19d48917503d000001';
		var weights, promiseError;

		beforeEach(function(done) {

			BodyWeightModel.deleteByDateRange(personId, startDate, endDate)
				.then(function() {
					return BodyWeightModel.add(personId, weightOneValue, weightOneDate);
				})
				.then(function() {
					return BodyWeightModel.add(personId, weightTwoValue, weightTwoDate);
				})
				.then(function() {
					return BodyWeightModel.getByDateRange(personId, startDate, endDate);
				})
				.then(function(items) {
					weights = items;
					done();
				},
				function(error) {
					promiseError = error;
					console.log(error.message);
					done();
				});
		});

		it('should have gotten weights', function() {
			expect(promiseError).to.be.undefined;
			expect(weights.length).to.equal(2);
			expect(weights[0].value).to.equal(weightOneValue);
			expect(weights[1].value).to.equal(weightTwoValue);
		});
	});

	describe('all weights by day', function() {
		var weightOneDate = '2012-02-01T12:22:33.444Z';
		var weightOneValue = 123;
		var weightTwoDate = '2012-02-01T14:33:11.321Z';
		var weightTwoValue = 213.32;
		var personId = '5170ab19d48917503d000001';
		var weights, promiseError;

		beforeEach(function(done) {

			BodyWeightModel.deleteAllOnDay(personId, weightOneDate)
				.then(function() {
					return BodyWeightModel.add(personId, weightOneValue, weightOneDate);
				})
				.then(function() {
					return BodyWeightModel.add(personId, weightTwoValue, weightTwoDate);
				})
				.then(function() {
					return BodyWeightModel.getAllOnDay(personId, weightTwoDate);
				})
				.then(function(items) {
					weights = items;
					done();
				},
				function(error) {
					promiseError = error;
					console.log(error.message);
					done();
				});
		});

		it('should have gotten weights', function() {
			expect(promiseError).to.be.undefined;
			expect(weights.length).to.equal(2);
			expect(weights[0].value).to.equal(weightOneValue);
			expect(weights[1].value).to.equal(weightTwoValue);
		});
	});


	describe('record by id', function() {
		var date = Date.now();
		var weight = 444;
		var personId = '5170ab19d48917503d000001';
		var addedWeight, searchedWeight, deletedWeight, inactiveWeight;
		var promiseError;

		beforeEach(function(done) {

			BodyWeightModel.deleteAllOnDay(personId, date)
				.then(function() {
					return BodyWeightModel.add(personId, weight, date);
				})
				.then(function() {
					return BodyWeightModel.getByDateRange(personId, date, date);
				})
				.then(function(items) {
					addedWeight = items[0];
					return BodyWeightModel.findById(addedWeight._id).exec();
				})
				.then(function(item) {
					searchedWeight = item;
					return BodyWeightModel.deleteById(addedWeight._id);
				})
				.then(function() {
					return BodyWeightModel.getById(addedWeight._id);
				})
				.then(function(item) {
					deletedWeight = item;
					return BodyWeightModel.findById(addedWeight._id).exec();
				})
				.then(function(item) {
					inactiveWeight = item;
					done();
				},
				function(error) {
					promiseError = error;
					done();
				});
		});

		it('should have added and deleted a record', function() {
			expect(promiseError, "Error: " + promiseError).to.equal(undefined);

			expect(addedWeight._id).to.not.equal(undefined);
			expect(addedWeight._id).to.not.equal(null);
			expect(addedWeight.value, "Added Weight value").to.equal(weight);

			expect(searchedWeight._id.toString(), 'Searched Weight id').to.equal(addedWeight._id.toString());
			expect(searchedWeight.value, "Searched Weight value").to.equal(weight);

			expect(deletedWeight, "Deleted Weight length").to.equal(null);

			expect(inactiveWeight._id.toString(), "Inactive Weight id").to.equal(addedWeight._id.toString());
			expect(inactiveWeight, "Inactive Weight").to.not.equal(undefined);
			expect(inactiveWeight.value, "Inactive Weight value").to.equal(weight);
			expect(inactiveWeight.active, "Inactive Weight active").to.equal(false);
		});

	});
});
