describe('person', function() {

	var person = require('../lib/person'),
		mongoose = require('mongoose'),
		expect = require('chai').expect,
		settings = require('./test-database-configuration');

	var connectionString = 'mongodb://' + settings.databaseServer + ':' + settings.databasePort + '/' + settings.databaseName;


	before(function(done) {
		mongoose.connect(connectionString);
		PersonModel.find({}).remove();
		PersonModel.ensureIndexes(function(error) {
			if (error) { console.log(error.message); }
		});
		mongoose.disconnect(function(){done();});
	})

	afterEach(function(done) {
		mongoose.disconnect(function(){done();});
	});

	beforeEach(function(done) {
		mongoose.connect(connectionString);
		PersonModel.ensureIndexes(function(error) {
			if (error) { console.log(error.message); }
			done();
		});
	});

	var PersonModel = person.getModel(mongoose);

	it('should get the collection name', function() {
		expect(PersonModel.getCollectionName()).to.equal('Person');
	});

	it('should get the model name', function() {
		expect(PersonModel.getModelName()).to.equal('Person');
	});

	it('should get a name object from a full name string', function() {
		var firstName = 'FirstName';
		var middleName = 'MiddleName';
		var lastName = 'LastName';
		var fullName = firstName + ' ' + middleName + ' ' + lastName;

		var name = PersonModel.getName(fullName);

		expect(name.first).to.equal(firstName);
		expect(name.middle).to.equal(middleName);
		expect(name.last).to.equal(lastName);
	});

	it('should get a person document', function(done) {
		var person = new PersonModel({
			name: {
				first: 'Mark',
				middle: 'Gilbert',
				last: 'Ott'
			},
			active: true,
			createdOn: new Date()
		});

		person.save(function(error) {
			expect(error).to.be.null;
			done();
		});
	});

	it('should not allow duplicates', function(done) {
		var person1 = new PersonModel({
			name: {
				first: 'Duplicate',
				last: 'Person'
			},
			active: true,
			createdOn: new Date()
		});

		var person2 = new PersonModel({
			name: {
				first: 'Duplicate',
				last: 'Person'
			},
			active: true,
			createdOn: new Date()
		});

		person1.save(function(error) {
			expect(error).to.be.null;
			person2.save(function(duplicateError) {
				expect(duplicateError).to.not.be.null;
				done();
			});
		});
	})

	it('should require a first name', function(done) {
		var person = new PersonModel({
			name: {
				middle: 'Gilbert',
				last: 'Ott'
			},
			active: true,
			createdOn: new Date()
		});

		person.save(function(error) {
			expect(error).to.be.an('object');
			done();
		});
	});

	it('should NOT require a middle name', function(done) {
		var person = new PersonModel({
			name: {
				first: 'Mark',
				last: 'Ott'
			},
			active: true,
			createdOn: new Date()
		});

		person.save(function(error) {
			expect(error).to.be.null;
			done();
		});
	});

	it('should require a last name', function(done) {
		var person = new PersonModel({
			name: {
				first: 'Mark',
				middle: 'Gilbert'
			},
			active: true,
			createdOn: new Date()
		});

		person.save(function(error) {
			expect(error).to.be.an('object');
			done();
		});
	});

	it('should default the active flag to true', function(done) {
		var person = new PersonModel({
			name: {
				first: 'Mark',
				middle: 'Gilbert',
				last: 'DefaultActive'
			},
			createdOn: new Date()
		});

		person.save(function(error) {
			expect(error).to.be.falsy;
			PersonModel.findOne({
				name: {
					first: 'Mark',
					middle: 'Gilbert',
					last: 'DefaultActive'
				}
			}, function(error, savedPerson) {
				expect(error).to.be.null;
				expect(savedPerson.active).to.equal(true);
				done();
			});
		});
	});

	it('should be able to override the default active flag', function(done) {
		var person = new PersonModel({
			name: {
				first: 'Mark',
				middle: 'Gilbert',
				last: 'FalseActive'
			},
			active: false,
			createdOn: new Date()
		});

		person.save(function(error) {
			expect(error).to.be.falsy;
			PersonModel.findOne({
				name: {
					first: 'Mark',
					middle: 'Gilbert',
					last: 'FalseActive'
				}
			}, function(error, savedPerson) {
				expect(error).to.be.null;
				expect(savedPerson.active).to.equal(false);
				done();
			});
		});
	});


//	it('should get a person by id', function(done) {
//
//	});
//		person.add(fullName)
//			.then(function(){
//				expect(person.name.first).to.equal('Mark6');
//				done();
//			},
//			function (error) {
//				expect(error).to.be.null;
//			});

//		var person = new Person(mongo, fullName, function(err, i) {
//			expect(err).to.be.null;
//
//			new Person(mongo, person._id.toString(), function(error, items) {
//				expect(error).to.be.null;
//				expect(items).to.be.an('object');
//				expect(items.name.first).to.equal(firstName);
//				expect(items.name.middle).to.equal(middleName);
//				expect(items.name.last).to.equal(lastName);
//				expect(person.name.first).to.equal(firstName);
//				expect(person.isValid).to.be.true;
//				done();
//			});
//		});
	});
//
//	it('should get a user by name', function(done) {
//		var person = new Person(mongo, fullName, function (error, items) {
//			expect(error).to.be.null;
//			expect(items).to.be.an('object');
//			expect(items.name.first).to.equal(firstName);
//			expect(items.name.middle).to.equal(middleName);
//			expect(items.name.last).to.equal(lastName);
//			expect(person.name.first).to.equal(firstName);
//			expect(person.isValid).to.be.true;
//			done();
//		});
//	});
//
//	it('should have added a new person', function(done) {
//		var person = new Person(mongo, fullName, function (error, item) {
//			expect(error).to.be.null;
//			expect(item.name.first).to.equal(firstName);
//			expect(person.isValid).to.be.true;
//			expect(person.active).to.be.true;
//			done();
//		});
//	});
//
//	it('should remove a person', function(done) {
//		var person = new Person(mongo, fullName, function (error, item) {
//			expect(error).to.be.null;
//			expect(item.name.first).to.equal(firstName);
//			expect(person.isValid).to.be.true;
//			expect(person.active).to.be.true;
//
//			person.delete(function() {
//				expect(person.isValid).to.be.false;
//				expect(person.active).to.be.false;
//
//				done();
//			});
//		});
//	});

//});